==> websocket-connector-0.0.3-sources.jar: br/pucrio/tecgraf/soma/websocketnotifier/application/service/KtableService.java <==
package br.pucrio.tecgraf.soma.websocketnotifier.application.service;

import br.pucrio.tecgraf.soma.serviceapi.configuration.ServiceConfiguration;

import br.pucrio.tecgraf.soma.websocketnotifier.application.configuration.Constants;
import br.pucrio.tecgraf.soma.websocketnotifier.model.WebsocketNotifierInfo;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.*;
import javax.ws.rs.core.Response;

@Service
public class KtableService {

    private final Logger logger = LoggerFactory.getLogger(KtableService.class);
    private final String KSQLDB_URL;
    private final String KAFKA_NOTIFICATION_TOPIC;

    @Autowired
    public KtableService(ServiceConfiguration serviceConfiguration) {
        this.KSQLDB_URL = serviceConfiguration.getValue(Constants.Config.KAFKA_KSQLDB_URL.option.getLongName());
        this.KAFKA_NOTIFICATION_TOPIC = serviceConfiguration.getValue(Constants.Config.KAFKA_NOTIFICATION_TOPIC.option.getLongName());
    }

    private String getLastUserOffsetKQuery(WebsocketNotifierInfo userInfo){
        return "{\"ksql\": \"select * from ktable_websocket_notifier where ROWKEY = '"+userInfo.getUser()+"|+|"+userInfo.getApplication()+"|+|"+userInfo.getTopicName()+"';\", \"streamsProperties\": {}}";
    }

    private String getKStreamCreateKSQL(){
        return "{ \"ksql\": \"CREATE STREAM websocket_notifier_stream (user VARCHAR, application VARCHAR, topic_name VARCHAR, offset BIGINT) WITH (KAFKA_TOPIC = \'"+this.KAFKA_NOTIFICATION_TOPIC+"\', VALUE_FORMAT=\'JSON\');\"," +
                "  \"streamsProperties\": {\"ksql.streams.auto.offset.reset\": \"earliest\"} }";
    }

    private String getKTableCreateKSQL(){
        return "{ \"ksql\": \"CREATE TABLE ktable_websocket_notifier AS SELECT user, application, topic_name, MAX(offset) AS offset FROM WEBSOCKET_NOTIFIER_STREAM GROUP BY user, application, topic_name;\"," +
                "  \"streamsProperties\": {\"ksql.streams.auto.offset.reset\": \"earliest\"} }";
    }

    private WebTarget getWebTarget(String uri){
        return ClientBuilder.newClient().target(uri);
    }

    protected Response makeRequest(String query, String endpoint, String path) {
        logger.info("makeRequest {} {}", endpoint, path);
        WebTarget target = this.getWebTarget(endpoint);
        Entity queryEntity =  Entity.json(query);
        Response response = target.path(path).request("application/vnd.ksql.v1+json").post(queryEntity);
        return response;
    }

    public WebsocketNotifierInfo getLastUserOffset(WebsocketNotifierInfo userInfo){
        String query = this.getLastUserOffsetKQuery(userInfo);
        try {
            logger.debug(query);
            Response response = makeRequest(query, this.KSQLDB_URL, "query");
            if (response.getStatus() != 200) {
                logger.error("Error during getLastUserOffset.", response.getEntity());
            }
            return parseJsonArray(userInfo, response.readEntity(String.class));
        } catch (ProcessingException e) {
            logger.error("Error during getLastUserOffset.", e);
        }
        return userInfo;
    }
    
    private boolean ktableDoesNotExists(Object response){
        if(response instanceof JsonObject
                && ((JsonObject) response).has("@type")
                && ((JsonObject) response).get("@type").getAsString().equals("statement_error")
                && ((JsonObject) response).get("error_code").getAsString().equals("40001")) {
            return true;
        }
        return false;
    }

    private boolean ktableReturnHasTuple(Object response){
        if(response instanceof JsonArray) {
            if (((JsonArray) response).size() > 1) {
                return true;
            }
        }
        return false;
    }

    private WebsocketNotifierInfo parseJsonArray(WebsocketNotifierInfo userInfo,  String response){
        JsonParser parser = new JsonParser();
        Object responseJson = parser.parse(response);
        if(ktableDoesNotExists(responseJson)){
            if(this.createKtableNotifier()) {
                return getLastUserOffset(userInfo);
            } else {
                logger.info("Websocket notification topic does not exist yet.");
                return userInfo;
            }
        }
        if(ktableReturnHasTuple(responseJson)){
            JsonObject row = ((JsonArray) responseJson).get(1).getAsJsonObject();
            userInfo.setOffset(row.get("row").getAsJsonObject().get("columns").getAsJsonArray().get(5).getAsInt());
            return userInfo;
        }
        logger.warn("Json return empty from KSQL query: {}", response);
        return userInfo;
    }

    private boolean createKSQLAccessStructure(String ksql){
        try {
            Response response = this.makeRequest(ksql, this.KSQLDB_URL, "ksql");
            logger.debug("Response create KSQL ACCESS STRUCTURE", response.readEntity(String.class));
            if (response.getStatus() != 200) {
                logger.error("Error while creating KSQL ACCESS STRUCTURE URI {} RESPONSE {} ", this.KSQLDB_URL, response);
                return false;
            }
            return true;
        } catch (ProcessingException e) {
            logger.error("Response create KSQL ACCESS STRUCTURE", e);
            return false;
        }
    }

    protected boolean createKtableNotifier() {
        logger.info("Creating KTABLE websocket notifier");
        return createKSQLAccessStructure(getKStreamCreateKSQL()) &&
                createKSQLAccessStructure(getKTableCreateKSQL());
    }

}

==> websocket-connector-1.0.0-sources.jar: br/pucrio/tecgraf/soma/websocketnotifier/application/service/KtableService.java <==
package br.pucrio.tecgraf.soma.websocketnotifier.application.service;

import br.pucrio.tecgraf.soma.serviceapi.configuration.ServiceConfiguration;

import br.pucrio.tecgraf.soma.websocketnotifier.application.configuration.Constants;
import br.pucrio.tecgraf.soma.websocketnotifier.model.WebsocketNotifierInfo;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import jakarta.ws.rs.ProcessingException;
import jakarta.ws.rs.client.*;
import jakarta.ws.rs.core.Response;

@Service
public class KtableService {

    private final Logger logger = LoggerFactory.getLogger(KtableService.class);
    private final String KSQLDB_URL;
    private final String KAFKA_NOTIFICATION_TOPIC;

    @Autowired
    public KtableService(ServiceConfiguration serviceConfiguration) {
        this.KSQLDB_URL = serviceConfiguration.getValue(Constants.Config.KAFKA_KSQLDB_URL.option.getLongName());
        this.KAFKA_NOTIFICATION_TOPIC = serviceConfiguration.getValue(Constants.Config.KAFKA_NOTIFICATION_TOPIC.option.getLongName());
    }

    private String getLastUserOffsetKQuery(WebsocketNotifierInfo userInfo){
        return "{\"ksql\": \"select * from ktable_websocket_notifier where ROWKEY = '"+userInfo.getUser()+"|+|"+userInfo.getApplication()+"|+|"+userInfo.getTopicName()+"';\", \"streamsProperties\": {}}";
    }

    private String getKStreamCreateKSQL(){
        return "{ \"ksql\": \"CREATE STREAM websocket_notifier_stream (user VARCHAR, application VARCHAR, topic_name VARCHAR, offset BIGINT) WITH (KAFKA_TOPIC = \'"+this.KAFKA_NOTIFICATION_TOPIC+"\', VALUE_FORMAT=\'JSON\');\"," +
                "  \"streamsProperties\": {\"ksql.streams.auto.offset.reset\": \"earliest\"} }";
    }

    private String getKTableCreateKSQL(){
        return """
                { "ksql": "CREATE TABLE ktable_websocket_notifier AS SELECT user, application, topic_name, MAX(offset) AS offset FROM WEBSOCKET_NOTIFIER_STREAM GROUP BY user, application, topic_name;",\
                  "streamsProperties": {"ksql.streams.auto.offset.reset": "earliest"} }\
                """;
    }

    private WebTarget getWebTarget(String uri){
        return ClientBuilder.newClient().target(uri);
    }

    protected Response makeRequest(String query, String endpoint, String path) {
        logger.info("makeRequest {} {}", endpoint, path);
        WebTarget target = this.getWebTarget(endpoint);
        Entity queryEntity =  Entity.json(query);
        Response response = target.path(path).request("application/vnd.ksql.v1+json").post(queryEntity);
        return response;
    }

    public WebsocketNotifierInfo getLastUserOffset(WebsocketNotifierInfo userInfo){
        String query = this.getLastUserOffsetKQuery(userInfo);
        try {
            logger.debug(query);
            Response response = makeRequest(query, this.KSQLDB_URL, "query");
            if (response.getStatus() != 200) {
                logger.error("Error during getLastUserOffset.", response.getEntity());
            }
            return parseJsonArray(userInfo, response.readEntity(String.class));
        } catch (ProcessingException e) {
            logger.error("Error during getLastUserOffset.", e);
        }
        return userInfo;
    }
    
    private boolean ktableDoesNotExists(Object response){
        if(response instanceof JsonObject object
                && object.has("@type")
                && object.get("@type").getAsString().equals("statement_error")
                && object.get("error_code").getAsString().equals("40001")) {
            return true;
        }
        return false;
    }

    private boolean ktableReturnHasTuple(Object response){
        if(response instanceof JsonArray array) {
            if (array.size() > 1) {
                return true;
            }
        }
        return false;
    }

    private WebsocketNotifierInfo parseJsonArray(WebsocketNotifierInfo userInfo,  String response){
        JsonParser parser = new JsonParser();
        Object responseJson = parser.parse(response);
        if(ktableDoesNotExists(responseJson)){
            if(this.createKtableNotifier()) {
                return getLastUserOffset(userInfo);
            } else {
                logger.info("Websocket notification topic does not exist yet.");
                return userInfo;
            }
        }
        if(ktableReturnHasTuple(responseJson)){
            JsonObject row = ((JsonArray) responseJson).get(1).getAsJsonObject();
            userInfo.setOffset(row.get("row").getAsJsonObject().get("columns").getAsJsonArray().get(5).getAsInt());
            return userInfo;
        }
        logger.warn("Json return empty from KSQL query: {}", response);
        return userInfo;
    }

    private boolean createKSQLAccessStructure(String ksql){
        try {
            Response response = this.makeRequest(ksql, this.KSQLDB_URL, "ksql");
            logger.debug("Response create KSQL ACCESS STRUCTURE", response.readEntity(String.class));
            if (response.getStatus() != 200) {
                logger.error("Error while creating KSQL ACCESS STRUCTURE URI {} RESPONSE {} ", this.KSQLDB_URL, response);
                return false;
            }
            return true;
        } catch (ProcessingException e) {
            logger.error("Response create KSQL ACCESS STRUCTURE", e);
            return false;
        }
    }

    protected boolean createKtableNotifier() {
        logger.info("Creating KTABLE websocket notifier");
        return createKSQLAccessStructure(getKStreamCreateKSQL()) &&
                createKSQLAccessStructure(getKTableCreateKSQL());
    }

}
